/*
    *
    * This file is a part of Libcprime.
    * Library for bookmarking, saving recent activites, managing settings for CuboCore Application Suite
	* Copyright 2019 CuboCore Group

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/


#include <QProcess>
#include <QFileInfo>
#include <QWidget>
#include <QUrl>

#include "infofunc.h"
#include "cvariables.h"
#include "settingsmanage.h"
#include "desktopfile.h"
#include "mimeutils.h"
#include "appopenfunc.h"
#include "applicationdialog.h"

/*
 * Open app using QProcess with argument.
 * appName : must need to pass exact app name
 * N.B. There is no checking method to check whether the app name is correct.
 */
void CPrime::AppOpenFunc::systemAppOpener(const QString &appName, const QStringList &arg)
{
    bool status = false;
    QProcess proc;

    if (!arg.isEmpty()) {
        status = proc.startDetached(appName, arg);
    } else {
        status = proc.startDetached(appName);
    }

    if (!status) {
        if (proc.error()) {
            qDebug() << "func(systemAppOpener) : Something gone wrong app not opened : args : " << appName << arg;
            return;
        }
    }

    qDebug() << "func(systemAppOpener) : Opening : args : " << appName << arg;
}

/*
 * Catagorize a path with the extention to open on specific app
 * path : Need a specific pathCPrimeCPrime
 * processOwner : Object which responsible to open the app
 */
void CPrime::AppOpenFunc::appOpenEngine(const QString &path, QObject *processOwner)
{
    QFileInfo file(path);

    if (!file.exists() || path.count() == 0) {
        CPrime::InfoFunc::messageEngine("File not exists...", CPrime::MessageType::Warning, static_cast<QWidget *>(processOwner));
        return;
    }

    QUrl url = QUrl::fromLocalFile(path);
    qDebug() << QString("\"" + path + "\"") << url.toLocalFile();
    QProcess::startDetached("xdg-open", QStringList() << QString(url.toLocalFile()));
}

/*
 * Open app (open the *.desktop file) based on category.
 */
void CPrime::AppOpenFunc::defaultAppEngine(CPrime::Category ctg, const QFileInfo &file, QObject *processOwner)
{
    settingsManage *sm = settingsManage::initialize();
    MimeUtils *mimeUtils = new MimeUtils;
    QString path(file.absoluteFilePath());
    QWidget *parent = static_cast<QWidget *>(processOwner);
    if (!parent) {
        parent = nullptr;
    }

    switch (ctg) {
        case CPrime::Category::FileManager: {
            // Needs to be the *.desktop name
            QString defaultFileManager = sm->value("CoreApps", "FileManager"); // selected File Manager name from settings.
            if (defaultFileManager.isEmpty() || defaultFileManager == "Nothing") {
                CPrime::InfoFunc::messageEngine("No default app selected!!!\n Select a default app to avoid this message", CPrime::MessageType::Warning, parent);
                QString appName(setDefaultApp());
                if(!appName.isEmpty()){
                    sm->setValue("CoreApps", "FileManager", appName);
                    defaultFileManager = appName;
                } else {
                    return;
                }
            }

            QString appName = defaultFileManager + ".desktop";
            QString desktopFilePath = CPrime::Variables::CC_System_AppPath() + appName;
            if (!QFileInfo(desktopFilePath).exists()) {
                CPrime::InfoFunc::messageEngine(QString("Selected file manager(%1) not found").arg(defaultFileManager), CPrime::MessageType::Warning, parent);
                return;
            }

            DesktopFile df = DesktopFile(desktopFilePath);
            mimeUtils->openInApp(df.getExec(), file, processOwner);

            // Show message
            CPrime::InfoFunc::messageEngine(defaultFileManager + " opening", CPrime::MessageType::Info, parent);

            break;
        }

        case CPrime::Category::MetadataViewer: {
            QString defaultMetadataViewer = sm->value("CoreApps", "MetadataViewer"); // selected MetadataViewer name from settings.
            if (defaultMetadataViewer.isEmpty() || defaultMetadataViewer == "Nothing") {
                CPrime::InfoFunc::messageEngine("No default app selected!!!\n Select a default app to avoid this message", CPrime::MessageType::Warning, parent);
                QString appName(setDefaultApp());
                if(!appName.isEmpty()){
                    sm->setValue("CoreApps", "MetadataViewer", appName);
                    defaultMetadataViewer = appName;
                } else {
                    return;
                }
            }

            QString appName = defaultMetadataViewer + ".desktop";
            QString desktopFilePath = CPrime::Variables::CC_System_AppPath() + appName;
            if (!QFileInfo(desktopFilePath).exists()) {
                CPrime::InfoFunc::messageEngine(QString("Selected metadate viewer (%1) not found").arg(defaultMetadataViewer), CPrime::MessageType::Warning, parent);
                return;
            }

            DesktopFile df = DesktopFile(desktopFilePath);

            // Open in app will add it at recent activity
            mimeUtils->openInApp(df.getExec(), file, processOwner);

            // Show message
            QString mess = defaultMetadataViewer + " opening." ;
            CPrime::InfoFunc::messageEngine(mess, CPrime::MessageType::Info, parent);

            break;
        }

        case CPrime::Category::SearchApp: {
            QString defaultSearchApp = sm->value("CoreApps", "SearchApp"); // selected SearchApp name from settings.
            if (defaultSearchApp.isEmpty() || defaultSearchApp == "Nothing") {
                CPrime::InfoFunc::messageEngine("No default app selected!!!\n Select a default app to avoid this message", CPrime::MessageType::Warning, parent);
                QString appName(setDefaultApp());
                if (!appName.isEmpty()) {
                    sm->setValue("CoreApps", "SearchApp", appName);
                    defaultSearchApp = appName;
                } else {
                    return;
                }
            }

            if ( defaultSearchApp == "corehunt" ) {
				QProcess::startDetached( defaultSearchApp, QStringList() << "--path" << path );

                // Show message
                QString mess = defaultSearchApp + " opening.";
                CPrime::InfoFunc::messageEngine(mess, CPrime::MessageType::Info, parent);

                return;
			}

            QString appName = defaultSearchApp + ".desktop";
            QString desktopFilePath = CPrime::Variables::CC_System_AppPath() + appName;
            if (!QFileInfo(desktopFilePath).exists()) {
                CPrime::InfoFunc::messageEngine(QString("Selected search app (%1) not found").arg(defaultSearchApp), CPrime::MessageType::Warning, parent);
                return;
            }

            DesktopFile df = DesktopFile(desktopFilePath);
            mimeUtils->openInApp(df.getExec(), file, processOwner);

            // Show message
            QString mess = defaultSearchApp + " opening.";
            CPrime::InfoFunc::messageEngine(mess, CPrime::MessageType::Info, parent);

            break;
        }

        case CPrime::Category::ImageEditor: {
            QString defaultImageEditor = sm->value("CoreApps", "ImageEditor"); // selected ImageEditor name from settings.
            if (defaultImageEditor.isEmpty() || defaultImageEditor == "Nothing") {
                CPrime::InfoFunc::messageEngine("No default app selected!!!\n Select a default app to avoid this message", CPrime::MessageType::Warning, parent);
                QString appName(setDefaultApp());
                if(!appName.isEmpty()){
                    sm->setValue("CoreApps", "ImageEditor", appName);
                    defaultImageEditor = appName;
                } else {
                    return;
                }
            }

            QString appName = defaultImageEditor + ".desktop";
            QString desktopFilePath = CPrime::Variables::CC_System_AppPath() + appName;
            if (!QFileInfo(desktopFilePath).exists()) {
                CPrime::InfoFunc::messageEngine(QString("Selected image editor(%1) not found").arg(defaultImageEditor), CPrime::MessageType::Warning, parent);
                return;
            }

            DesktopFile df = DesktopFile(desktopFilePath);
            mimeUtils->openInApp(df.getExec(), file, processOwner);

            // Show message
            QString mess = defaultImageEditor + " opening.";
            CPrime::InfoFunc::messageEngine(mess, CPrime::MessageType::Info, parent);

            break;
        }

        case CPrime::Category::Terminal: {

            QString defaultTerminal = sm->value("CoreApps", "Terminal"); // selected terminal name from settings.
            if (defaultTerminal.isEmpty() || defaultTerminal == "Nothing") {
                CPrime::InfoFunc::messageEngine("No default app selected!!!\n Select a default app to avoid this message", CPrime::MessageType::Warning, parent);
                QString appName(setDefaultApp());
                if(!appName.isEmpty()){
                    sm->setValue("CoreApps", "Terminal", appName);
                    defaultTerminal = appName;
                } else {
                    return;
                }
            }

			/* Terminals which take --working-directory switch */
			QStringList longwd = { "coreterminal", "gnome-terminal", "terminator", "mate-terminal", "xfce4-terminal" };

			/* Terminals which take --workdir switch */
			QStringList shortwd = { "qterminal", "konsole", "nbterminal" };

			/* Terminals which do not accept work-dir switch */
			QStringList badwd = { "xterm" };

			/* Special baby of the group: LXTerminal which wants --working-directory=WD */
            QProcess proc;
			 if ( defaultTerminal == "lxterminal" )
                proc.startDetached( defaultTerminal, QStringList() << "--working-directory=" + path );

			else if ( longwd.contains( defaultTerminal ) )
                proc.startDetached( defaultTerminal, QStringList() << "--working-directory" << path );

			else if ( shortwd.contains( defaultTerminal ) )
                proc.startDetached( defaultTerminal, QStringList() << "--workdir" << path );

			else
                proc.startDetached( defaultTerminal, QStringList() << "-e"  << QString( "cd %1 & %2" ).arg( path ).arg( "/bin/bash" ) );

             if (!proc.error()) {
                 // Show message
                 QString mess = defaultTerminal + " opening.";
                 CPrime::InfoFunc::messageEngine(mess, CPrime::MessageType::Info, parent);
             }

            break;
        }
    }
}

QString CPrime::AppOpenFunc::setDefaultApp()
{
    // Select application in the dialog
    ApplicationDialog *dialog = new ApplicationDialog(nullptr);
    QString appName;
    if (dialog->exec()) {
        if (dialog->getCurrentLauncher().compare("") != 0) {
            appName = dialog->getCurrentLauncher();
        }
    } else {
        CPrime::InfoFunc::messageEngine("No default app selected!!!", CPrime::MessageType::Warning, nullptr);
        appName = "";
    }

    return appName;
}
