/*
    *
    * This file is a part of Libcprime.
    * Library for bookmarking, saving recent activites, managing settings for CuboCore Application Suite
	* Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/


#include <QWidget>
#include <QFileInfo>
#include <QProcess>
#include <QMimeDatabase>
#include <QMimeType>
#include <QTextStream>
#include <QUrl>
#include <QDirIterator>

#include "properties.h"
#include "desktopfile.h"
#include "cprime.h"
#include "mimeutils.h"
#include "validityfunc.h"

/**
 * @brief Creates mime utils
 * @param parent
 */
MimeUtils::MimeUtils(QObject *parent) : QObject(parent)
{
    defaultsFileName = CPrime::Variables::CC_CoreApps_MimeFilePath().remove(QDir::homePath());
    defaults = new Properties();
    loadDefaults();
}

/**
 * @brief Loads list of default applications for mimes
 * @return properties with default applications
 */
void MimeUtils::loadDefaults()
{
	if ( not QFileInfo( QDir::home().filePath( defaultsFileName ) ).exists() )
		generateDefaults();

    defaults->load(QDir::homePath() + defaultsFileName, "Default Applications");
    defaultsChanged = false;
}

/**
 * @brief Destructor
 */
MimeUtils::~MimeUtils()
{
    delete defaults;
}

/**
 * @brief Returns list of mime types
 * @return list of available mimetypes
 */
QStringList MimeUtils::getMimeTypes() const
{
    // Check whether file with mime descriptions exists
    QFile file("/usr/share/mime/types");

    if (!file.exists()) {
        return QStringList();
    }

    // Try to open file
    if (!file.open(QFile::ReadOnly)) {
        return QStringList();
    }

    // Read mime types
    QStringList result;
    QTextStream stream(&file);

    while (!stream.atEnd()) {
        result.append(stream.readLine());
    }

    file.close();
    return result;
}

/**
 * @brief Opens file in a default application
 * @param file
 * @param processOwner
 */
void MimeUtils::openInApp(const QFileInfo &file, QObject *processOwner)
{
    QMimeDatabase mimetype;
    QString mime = mimetype.mimeTypeForFile(file.filePath()).name();
    QString app = defaults->value(mime).toString().split(";").first();

    if (!app.isEmpty()) {
        DesktopFile df = DesktopFile("/usr/share/applications/" + app);
        openInApp(df.getExec(), file, processOwner);
    } else {
        // Function from utilities.cpp
        CPrime::InfoFunc::messageEngine(tr("No default application for mime:\n %1!").arg(mime), CPrime::MessageType::Warning, static_cast<QWidget *>(processOwner));
    }
}

/**
 * @brief Opens file in a given application
 * @param exe name of application to be executed
 * @param file to be opened in executed application
 * @param processOwner process owner (default NULL)
 */
void MimeUtils::openInApp(QString exe, const QFileInfo &file, QObject *processOwner)
{
    /*
    // This is not the right the solution, but qpdfview won't start otherwise
    // TODO: Repair it correctly
    if (exe.contains("qpdfview")) {
        exe = "qpdfview";
    }

    // Separate application name from its arguments
    QStringList split = exe.split(" ");
    QString name = split.takeAt(0);
    QString args = split.join(" ");
    // Get relative path
    //args = args.split(QDir::separator()).last();

    // Replace parameters with file name. If there are no parameters simply append
    // file name to the end of argument list
    if (args.contains("%f", Qt::CaseInsensitive)) {
        args.replace("%f", file.filePath(), Qt::CaseInsensitive);
    } else if (args.contains("%u", Qt::CaseInsensitive)) {
        args.replace("%u", file.filePath(), Qt::CaseInsensitive);
    } else {
        args.append(args.isEmpty() ? "" : " ");
        args.append("\"" + file.filePath() + "\"");
    }

    // Function from utilities.cpp
    CPrime::InfoFunc::saveToRecent(name, args);

    // Start application
    QProcess *myProcess = new QProcess(processOwner);

    QString tArg = args;

    if (args.remove(file.filePath()).contains(" ")) {
        args.append(file.filePath());
        if (args.remove(file.filePath()).split(" ").count()) {
            args.append(file.filePath());
            myProcess->startDetached(name, QStringList() << QString(args.split(" ").at(0)) << args.split(" ").at(1));
        }
    } else {
        myProcess->startDetached(name, QStringList() << tArg);
    }

    myProcess->waitForFinished(1000);
    //myProcess->terminate();
    */

    QStringList inputfiles;
    inputfiles << file.absoluteFilePath();

    QString exec = exe;
    QString name = exec.split(" ").at(0);

    // Does the app need the input files in URL or File syntax?
    bool URLsyntax = (exec.contains("%u") || exec.contains("%U"));

    // Adjust the input file formats as needed
    for (int i = 0; i < inputfiles.length(); i++) {
        bool url = inputfiles[i].startsWith("www") || inputfiles[i].contains("://");
        // Run it through the QUrl class to catch/fix any URL syntax issues
        if(URLsyntax){
            if (inputfiles[i].startsWith("mailto:")) {
                // don't touch this syntax - already formatted
            } else if (url) {
                inputfiles[i] = QUrl(inputfiles[i]).url();
            } else {
                inputfiles[i] = QUrl::fromLocalFile(inputfiles[i]).url();
            }
        } else {
            if (url) {
                inputfiles[i] = QUrl(inputfiles[i]).toLocalFile();
            } else {
                inputfiles[i] = inputfiles[i];
            } //QUrl::fromLocalFile(inputfiles[i]).toLocalFile(); }
        }
    }

    inputfiles.removeAll(""); //just in case any empty ones get through

    // Now to the exec replacements as needed
    if (exec.contains("%f")) {
        if (inputfiles.isEmpty()) {
            exec.replace("%f", "");
        } else {
            exec.replace("%f", "\"" + inputfiles.first() + "\""); //Note: can only take one input
        }
    } else if (exec.contains("%F")) {
        if (inputfiles.isEmpty()) {
            exec.replace("%F", "");
        } else {
            exec.replace("%F", "\"" + inputfiles.join("\" \"") + "\"");
        }
    }
    if (exec.contains("%u")) {
        if (inputfiles.isEmpty()) {
            exec.replace("%u", "");
        } else {
            exec.replace("%u", "\"" + inputfiles.first() + "\""); //Note: can only take one input
        }
    } else if (exec.contains("%U")) {
        if (inputfiles.isEmpty()) {
            exec.replace("%U", "");
        } else {
            exec.replace("%U", "\"" + inputfiles.join("\" \"") + "\"");
        }
    }

    //Sanity check for known Local/URL syntax issues from some apps
    if (!URLsyntax && exec.contains("%20")) {
        exec.replace("%20", " ");
    }

    //Clean up any leftover "Exec" field codes (should have already been replaced earlier)
    if (exec.contains("%")) {
        exec = exec.remove("%U").remove("%u").remove("%F").remove("%f").remove("%i").remove("%c").remove("%k");
    }

    // Save it to recent activity
    CPrime::InfoFunc::saveToRecent(name, file.absoluteFilePath());

    // Start the process
    QString argument = exec.simplified();
    QProcess *myProcess = new QProcess(processOwner);
    myProcess->startDetached(argument);
}

/**
 * @brief Returns list of available applications
 * @return application list
 */
QList<DesktopFile> MimeUtils::getApplications()
{
    QList<DesktopFile> apps;
    QDirIterator it("/usr/share/applications", QStringList("*.desktop"),
                    QDir::Files | QDir::NoDotAndDotDot,
                    QDirIterator::Subdirectories);

    while (it.hasNext()) {
        it.next();
        apps.append(DesktopFile(it.filePath()));
    }

    return apps;
}

/**
 * @brief Sets defaults file name (name of file where defaults are stored)
 * @param fileName
 */
void MimeUtils::setDefaultsFileName(const QString &fileName)
{
    this->defaultsFileName = fileName;
    loadDefaults();
}
//---------------------------------------------------------------------------

/**
 * @brief Returns defaults file name
 * @return name of file where defaults are stored
 */
QString MimeUtils::getDefaultsFileName() const
{
    return defaultsFileName;
}

/**
 * @brief Generates default application-mime associations
 */
void MimeUtils::generateDefaults()
{
    // Load list of applications
    QList<DesktopFile> apps = MimeUtils::getApplications();
    QStringList names;

    // Find defaults; for each application...
    // ------------------------------------------------------------------------
    foreach (DesktopFile a, apps) {

        // For each mime of current application...
        QStringList mimes = a.getMimeType();

        foreach (QString mime, mimes) {

            // Current app name
            QString name = a.getPureFileName() + ".desktop";
            names.append(name);

            // If current mime is not mentioned in the list of defaults, add it
            // together with current application and continue
            if (!defaults->contains(mime)) {
                defaults->set(mime, name);
                defaultsChanged = true;
                continue;
            }

            // Retrieve list of default applications for current mime, if it does
            // not contain current application, add this application to list
            QStringList appNames = defaults->value(mime).toString().split(";");

            if (!appNames.contains(name)) {
                appNames.append(name);
                defaults->set(mime, appNames.join(";"));
                defaultsChanged = true;
            }
        }
    }

    // Delete dead defaults (non existing apps)
    // ------------------------------------------------------------------------
    foreach (QString key, defaults->getKeys()) {
        QStringList tmpNames1 = defaults->value(key).toString().split(";");
        QStringList tmpNames2 = QStringList();

        foreach (QString name, tmpNames1) {
            if (names.contains(name)) {
                tmpNames2.append(name);
            }
        }

        if (tmpNames1.size() != tmpNames2.size()) {
            defaults->set(key, tmpNames2.join(";"));
            defaultsChanged = true;
        }
    }

    // Save defaults if changed
    saveDefaults();
}

/**
 * @brief Sets default mime association
 * @param mime mime name
 * @param apps list of applications (desktop file names)
 */
void MimeUtils::setDefault(const QString &mime, const QStringList &apps)
{
    QString value = apps.join(";");

    if (value.compare(defaults->value(mime, "").toString()) != 0) {
        defaults->set(mime, value);
        defaultsChanged = true;
    }
}

/**
 * @brief Returns default applications for given mime
 * @param mime
 * @return list of default applications name
 */
QStringList MimeUtils::getDefault(const QString &mime) const
{
    return defaults->value(mime).toString().split(";");
}

/**
 * @brief Saves defaults
 */
void MimeUtils::saveDefaults()
{
    if (defaultsChanged) {
        defaults->save(QDir::homePath() + defaultsFileName, "Default Applications");
        defaultsChanged = false;
    }
}
