/*
    *
    * This file is a part of Libcprime.
    * Library for bookmarking, saving recent activites, managing settings for CuboCore Application Suite
	* Copyright 2019 CuboCore Group
    *

    *
    * This program is free software; you can redistribute it and/or modify
    * it under the terms of the GNU General Public License as published by
    * the Free Software Foundation; either version 3 of the License, or
    * (at your option) any later version.
    *

    *
    * This program is distributed in the hope that it will be useful,
    * but WITHOUT ANY WARRANTY; without even the implied warranty of
    * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    * GNU General Public License for more details.
    *

    *
    * You should have received a copy of the GNU General Public License
    * along with this program; if not, vsit http://www.gnu.org/licenses/.
    *
*/

#include "shareit.h"
#include "ui_shareit.h"

#include <QIcon>
#include <QStorageInfo>
#include <QListWidgetItem>

#include <cprime/ioprocesses.h>
#include <cprime/settingsmanage.h>
#include <cprime/desktopfile.h>
#include <cprime/cvariables.h>
#include <cprime/applicationdialog.h>
#include <cprime/appopenfunc.h>
#include <cprime/pinmanage.h>
#include <cprime/btfiletransfer/remoteselector.h>


shareit::shareit(QWidget *parent):QDialog(parent),ui(new Ui::shareit)
{
    ui->setupUi(this);

    setWindowIcon(QIcon::fromTheme("document-send"));

    settingsManage *sm = settingsManage::initialize();
    QSize listViewIconSize = sm->value("CoreApps", "ListViewIconSize");
    QSize iconViewIconSize = sm->value("CoreApps", "IconViewIconSize");
    ui->fApps->setIconSize(listViewIconSize);
    ui->drives->setIconSize(listViewIconSize);
    ui->network->setIconSize(listViewIconSize);
    ui->speedDial->setIconSize(iconViewIconSize);
    ui->homeFolder->setIconSize(iconViewIconSize);

    connect( ui->network, &QListWidget::itemActivated, this, &shareit::networkTransfer );
    connect( ui->close, SIGNAL(clicked()) , this, SLOT(close()));
}

shareit::~shareit()
{
    delete ui;
}

void shareit::setFilePath(QString filePath)
{
    m_filePath = filePath;
}

void shareit::populateFavouriteApps()
{
    settingsManage *sm = settingsManage::initialize();
    ui->fApps->clear();
    QStringList apps = sm->value("CoreApps", "Favourite");
    foreach (QString app, apps) {
        DesktopFile file = DesktopFile(CPrime::Variables::CC_System_AppPath() + "/" + app + ".desktop");
        QIcon i(ApplicationDialog::searchAppIcon(file));
        QListWidgetItem *item = new QListWidgetItem(file.getName());
        item->setData(Qt::UserRole, app);
        item->setIcon(i);
        item->setText(file.getName());
        ui->fApps->insertItem(0, item);
    }
}

void shareit::populateMountedDrives()
{
    foreach (QStorageInfo info, QStorageInfo::mountedVolumes()) {
        if (info.device() == QByteArray("tmpfs")) {
            continue;
        }

        QListWidgetItem *item = new QListWidgetItem(info.displayName());
        item->setData(Qt::UserRole, info.rootPath());
        item->setIcon(QIcon::fromTheme("drive-harddisk"));
        ui->drives->insertItem(0, item);
    }
}

void shareit::populateFolders()
{
    QIcon folder = QIcon::fromTheme("folder");

    // Speed Dial
    pinmanage pins;
    QStringList pinList = pins.getPinNames("Speed Dial");
    foreach (QString f, pinList) {
        QString path = pins.pinPath("Speed Dial", f);

        if (QFileInfo(path).isFile()) {
            continue;
        }

        QListWidgetItem *item = new QListWidgetItem(f);
        item->setData(Qt::UserRole, pins.pinPath("Speed Dial", f));
        item->setIcon(folder);
        ui->speedDial->addItem(item);
    }

    // Home folders (Desktop, documents, downloads, music, pictures, videos)
    QStringList fList;
    fList << "Desktop" << "Documents" << "Downloads" << "Music" << "Pictures" << "Videos";
    QString homePath = QDir::homePath();
    foreach (QString str, fList) {
        QListWidgetItem *item = new QListWidgetItem(str);
        item->setData(Qt::UserRole, homePath + "/" + str);
        item->setIcon(folder);
        ui->homeFolder->addItem(item);
    }
}

void shareit::on_fApps_itemActivated(QListWidgetItem *item)
{
    CPrime::AppOpenFunc::systemAppOpener(item->data(Qt::UserRole).toString(), QStringList() << m_filePath);
}

void shareit::on_drives_itemActivated(QListWidgetItem *item)
{
    QString driveFolder = toDriveFolder(item->data(Qt::UserRole).toString());
    qDebug() << driveFolder;
    if (driveFolder.count()) {
        IODialog::copy( m_filePath, item->data(Qt::UserRole).toString() );
    }
}

void shareit::on_speedDial_itemActivated(QListWidgetItem *item)
{
    IODialog::copy(m_filePath, item->data(Qt::UserRole).toString());
}

void shareit::on_homeFolder_itemActivated(QListWidgetItem *item)
{
    IODialog::copy(m_filePath, item->data(Qt::UserRole).toString());
}

void shareit::networkTransfer(QListWidgetItem *item) {

	if ( item->text() != "Bluetooth" ) {
		return;
	}

	QBluetoothLocalDevice *dev = new QBluetoothLocalDevice( this );
	if ( dev->hostMode() == QBluetoothLocalDevice::HostPoweredOff ) {
		QMessageBox::warning(
			this,
			"Bluetooth powered off",
			"Your bluetooth adapter is powered off. Please switch it on and try again."
		);

		return;
	}

    RemoteSelector d( QStringList() << m_filePath );
    d.startDiscovery();
    d.exec();
}

QString shareit::toDriveFolder(QString driveMountPoint)
{
    QDialog *driveFolder = new QDialog(this);
    driveFolder->setWindowIcon(this->windowIcon());
    driveFolder->setWindowTitle("Select folder");
    driveFolder->resize(300, 500);
    QListWidget *dFolders = new QListWidget;
    dFolders->setFrameShape(QListWidget::NoFrame);
    dFolders->setFrameShadow(QListWidget::Plain);
    dFolders->setMovement(QListWidget::Static);
    dFolders->setUniformItemSizes(true);
    dFolders->setWordWrap(true);
    dFolders->setSortingEnabled(true);
    QVBoxLayout *vBoxLayout = new QVBoxLayout;
    vBoxLayout->setContentsMargins(0, 0, 0, 0);
    vBoxLayout->addWidget(dFolders);
    driveFolder->setLayout(vBoxLayout);

    QListWidgetItem *item = nullptr;
    QIcon folder = QIcon::fromTheme("folder");
    QDirIterator iterator(driveMountPoint, QDir::Dirs | QDir::NoDotDot);
    while (iterator.hasNext()) {
        QString temp = iterator.next();
        item = new QListWidgetItem(temp);
        item->setData(Qt::UserRole, temp);
        item->setIcon(folder);
        dFolders->addItem(item);
    }

    QString selectedFolder = nullptr;
    connect(dFolders, &QListWidget::itemActivated, [&selectedFolder, driveFolder](QListWidgetItem *item) {
        selectedFolder = item->data(Qt::UserRole).toString();
        driveFolder->close();
    });

    if (driveFolder->exec() == 0) {

    }

    return selectedFolder;
}

/**
 * @brief Call shareit dialog from any apps by giving the path to share.
 * @param currentPath : Path which needs to be shared.
 */
void shareit::callShareIt(QWidget *parent, const QString &currentPath)
{
    shareit *shareIt = new shareit(parent);
    shareIt->setFilePath(currentPath);
    shareIt->populateFavouriteApps();
    shareIt->populateMountedDrives();
    shareIt->populateFolders();
    CPrime::InfoFunc::messageEngine( "Select an item to Share.", CPrime::Info, nullptr );

    if (shareIt->exec() == 0) {
    }
}
