/* libnotify-qt - library for sending notifications implemented in Qt
 * Copyright (C) 2010-2011 Vojtech Drbohlav <vojta.d@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "systemnotification.h"

#include <QDBusConnection>
#include <QDBusInterface>
#include <QDebug>

#include "orgfreedesktopnotificationsinterface.h"

bool systemNotification::s_isInitted = false;
QString systemNotification::s_appName;
org::freedesktop::Notifications * systemNotification::s_notifications = nullptr;

bool systemNotification::init(const QString & appName)
{
    if(!QDBusConnection::sessionBus().isConnected())
        return false;

    s_notifications = new org::freedesktop::Notifications("org.freedesktop.Notifications",
                                                          "/org/freedesktop/Notifications",
                                                          QDBusConnection::sessionBus());

    s_appName = appName;
    s_isInitted = s_notifications->isValid();
    return s_isInitted;
}

void systemNotification::uninit()
{
    s_isInitted = false;

    delete s_notifications;
}

bool systemNotification::isInitted()
{
    return s_isInitted;
}

const QString & systemNotification::getAppName()
{
    return s_appName;
}

QStringList systemNotification::getServerCaps()
{
    QDBusPendingReply<QStringList> reply = s_notifications->getCapabilities();
    reply.waitForFinished();

    if(reply.isValid())
    {
        return reply.argumentAt(0).toStringList();
    }

    return QStringList();
}

bool systemNotification::getServerInfo(QString & name, QString & vendor, QString & version)
{
    QDBusPendingReply<QString, QString, QString> reply = s_notifications->getServerInformation();
    reply.waitForFinished();

    if(reply.isValid())
    {
        name = reply.argumentAt(SERVER_INFO_NAME).toString();
        vendor = reply.argumentAt(SERVER_INFO_VENDOR).toString();
        version = reply.argumentAt(SERVER_INFO_VERSION).toString();
        return true;
    }

    return false;
}

systemNotification::systemNotification(const QString & summary, const QString & body, const QString & iconName, QObject * parent) :
    QObject(parent),
    m_id(0),
    m_summary(summary),
    m_body(body),
    m_iconName(iconName),
    m_timeout(5000),
    m_autoDelete(true)
{
    setUrgency(NOTIFICATION_URGENCY_NORMAL);

    // TODO: i should do this somehow in org::freedesktop::Notifications imho
    QDBusConnection::sessionBus().connect(QString(), QString(), org::freedesktop::Notifications::staticInterfaceName(),
                                          "NotificationClosed", this, SLOT(onNotificationClosed(quint32, quint32)));
    QDBusConnection::sessionBus().connect(QString(), QString(), org::freedesktop::Notifications::staticInterfaceName(),
                                          "ActionInvoked", this, SLOT(onActionInvoked(quint32,QString)));
}

systemNotification::~systemNotification()
{
    disconnect(s_notifications);
}

bool systemNotification::show()
{
    QDBusPendingReply<quint32> reply = s_notifications->notify(getAppName(), m_id, m_iconName, m_summary, m_body,
                                                               m_actions, m_hints, m_timeout);
    if(m_id == 0)
    {
        reply.waitForFinished();
        if(!reply.isValid())
            return false;

        m_id = reply.argumentAt(0).toInt();
    }

    return true;
}

void systemNotification::setSummary(const QString & summary)
{
    m_summary = summary;
}

void systemNotification::setBody(const QString & body)
{
    m_body = body;
}

void systemNotification::setIconName(const QString & iconName)
{
    m_iconName = iconName;
}

void systemNotification::setTimeout(qint32 timeout)
{
    m_timeout = timeout;
}

void systemNotification::setUrgency(NotificationUrgency urgency)
{
    setHintByte("urgency", urgency);
}

void systemNotification::setCategory(const QString & category)
{
    setHintString("category", category);
}

/*void Notification::setIconFromPixmap(const QPixmap & pixmap)
{
    setHintByteArray("image_data",pixmap.t);
}*/

void systemNotification::setLocation(qint32 x, qint32 y)
{
    setHintInt32("x", x);
    setHintInt32("y", y);
}

void systemNotification::setHint(const QString & key, const QVariant & value)
{
    m_hints.insert(key, value);
}

void systemNotification::setHintInt32(const QString & key, qint32 value)
{
    m_hints.insert(key, value);
}

void systemNotification::setHintDouble(const QString & key, double value)
{
    m_hints.insert(key, value);
}

void systemNotification::setHintString(const QString & key, const QString & value)
{
    m_hints.insert(key, value);
}

void systemNotification::setHintByte(const QString & key, char value)
{
    setHintByteArray(key, QByteArray(1, value));
}

void systemNotification::setHintByteArray(const QString & key, const QByteArray & value)
{
    m_hints.insert(key, value);
}

void systemNotification::clearHints()
{
    m_hints.clear();
}

void systemNotification::addAction(const QString & actionKey, const QString & label)
{
    m_actions << actionKey << label;
}

void systemNotification::clearActions()
{
    m_actions.clear();
}

bool systemNotification::close()
{
    QDBusPendingReply<> reply = s_notifications->closeNotification(m_id);
    reply.waitForFinished();
    return reply.isValid();
}

bool systemNotification::autoDelete() const
{
    return m_autoDelete;
}

void systemNotification::setAutoDelete(bool autoDelete)
{
    m_autoDelete = autoDelete;
}

void systemNotification::onNotificationClosed(quint32 id, quint32 reason)
{
    if(m_id == id)
    {
        emit closed(reason);

        if(m_autoDelete)
            deleteLater();
    }
}

void systemNotification::onActionInvoked(quint32 id, const QString & actionKey)
{
    if(m_id == id)
    {
        emit actionInvoked(actionKey);
    }
}
